﻿using System;

namespace UMK.WzorceStrukturalne
{
    #region Urządzenia
    public interface IUrządzenieRTV
    {
        void Włącz();
        void Wyłącz();
        void UstawKanał(int numerKanału);
    }

    public class Telewizor : IUrządzenieRTV
    {
        public void Włącz()
        {
            Console.WriteLine("Telewizor zostal wlaczony");
        }
        public void Wyłącz()
        {
            Console.WriteLine("Telewizor zostal wylaczony");
        }
        public void UstawKanał(int numerKanału)
        {
            Console.WriteLine("Kanal w telewizorze zmieniony na: " + numerKanału);
        }
    }

    public class Radio : IUrządzenieRTV
    {
        public void Włącz()
        {
            Console.WriteLine("Radio zostalo wlaczone");
        }
        public void Wyłącz()
        {
            Console.WriteLine("Radio zostalo wylaczone");
        }
        public void UstawKanał(int numerKanału)
        {
            Console.WriteLine("Kanal w radiu zmieniony na: " + numerKanału);
        }
    }
    #endregion

    class Pilot //most
    {
        private int bieżącyKanał;
        private int poprzedniKanał;
        protected IUrządzenieRTV urządzenie;

        public Pilot(IUrządzenieRTV urządzenie)
        {
            this.urządzenie = urządzenie;
            bieżącyKanał = -1;
            poprzedniKanał = -1;
        }

        public void UstawKanał(int nowyKanał)
        {
            poprzedniKanał = bieżącyKanał;
            bieżącyKanał = nowyKanał;
            urządzenie.UstawKanał(bieżącyKanał);
        }

        public void Włącz()
        {
            urządzenie.Włącz();
            UstawKanał(1);
        }

        public void Wyłącz()
        {
            urządzenie.Włącz();
        }
        
        public void PrzywróćPoprzedniKanał()
        {
            UstawKanał(poprzedniKanał);
        }
    }

    /*
    class PilotZespolony //most, choć zespolony
    {
        private int bieżącyKanał;
        private int poprzedniKanał;
        protected IUrządzenieRTV[] urządzenia;

        public PilotZespolony(params IUrządzenieRTV[] urządzenia)
        {
            this.urządzenia = urządzenia;
            bieżącyKanał = -1;
            poprzedniKanał = -1;
        }

        public void UstawKanał(int nowyKanał)
        {
            poprzedniKanał = bieżącyKanał;
            bieżącyKanał = nowyKanał;
            foreach(IUrządzenieRTV urządzenie in urządzenia)
                urządzenie.UstawKanał(bieżącyKanał);
        }

        public void Włącz()
        {
            foreach (IUrządzenieRTV urządzenie in urządzenia)
                urządzenie.Włącz();
            UstawKanał(1);
        }

        public void Wyłącz()
        {
            foreach (IUrządzenieRTV urządzenie in urządzenia)
                urządzenie.Włącz();
        }

        public void PrzywróćPoprzedniKanał()
        {
            UstawKanał(poprzedniKanał);
        }
    }
    */

    class PilotZespolony //pełnomocnik
    {
        private int bieżącyKanał;
        private int poprzedniKanał;
        protected IUrządzenieRTV[] urządzenia;

        public PilotZespolony()
        {
            urządzenia = new IUrządzenieRTV[]
            {
                new Radio(),
                new Telewizor()
            };
            
            bieżącyKanał = -1;
            poprzedniKanał = -1;
        }

        public IUrządzenieRTV[] Urządzenia
        {
            get
            {
                if (urządzenia == null) throw new Exception("Brak urządzeń");
                else return urządzenia;
            }
        }

        public void UstawKanał(int nowyKanał)
        {
            poprzedniKanał = bieżącyKanał;
            bieżącyKanał = nowyKanał;
            foreach (IUrządzenieRTV urządzenie in urządzenia)
                urządzenie.UstawKanał(bieżącyKanał);
        }

        public void Włącz()
        {
            foreach (IUrządzenieRTV urządzenie in urządzenia)
                urządzenie.Włącz();
            UstawKanał(1);
        }

        public void Wyłącz()
        {
            foreach (IUrządzenieRTV urządzenie in urządzenia)
                urządzenie.Włącz();
        }

        public void PrzywróćPoprzedniKanał()
        {
            UstawKanał(poprzedniKanał);
        }
    }
}
